import matplotlib
matplotlib.use('Agg') 
import pytest
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import os
import sys
import tempfile
from sklearn.neighbors import KNeighborsClassifier
from sklearn.datasets import make_classification

# Add the src directory to the Python path
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

# Import the functions to test
from banknote_utils.modeling_utils import (
    evaluate_knn_cv,
    plot_knn_cv,
    train_knn_model,
    evaluate_model
)

# Create a fixture for test data
@pytest.fixture
def test_data():
    # Generate a simple classification dataset
    X, y = make_classification(
        n_samples=100,
        n_features=4,
        n_informative=2,
        n_redundant=0,
        random_state=42
    )
    
    # Convert to pandas DataFrame and Series (like our actual data)
    X_df = pd.DataFrame(X, columns=[f'feature_{i}' for i in range(X.shape[1])])
    y_series = pd.Series(y, name='class')
    
    # Split data (70% train, 30% test)
    train_indices = np.random.choice(len(X), size=int(0.7 * len(X)), replace=False)
    test_indices = np.array([i for i in range(len(X)) if i not in train_indices])
    
    X_train = X_df.iloc[train_indices]
    y_train = y_series.iloc[train_indices]
    X_test = X_df.iloc[test_indices]
    y_test = y_series.iloc[test_indices]
    
    return {
        'X_train': X_train,
        'y_train': y_train,
        'X_test': X_test,
        'y_test': y_test
    }


class TestEvaluateKnnCV:
    def test_basic_functionality(self, test_data):
        """Test that evaluate_knn_cv returns the expected structure."""
        neighbors, cv_scores, best_k = evaluate_knn_cv(
            test_data['X_train'], 
            test_data['y_train'],
            k_range=range(1, 5)
        )
        
        # Check types and shapes
        assert isinstance(neighbors, list)
        assert isinstance(cv_scores, list)
        assert isinstance(best_k, int)
        assert len(neighbors) == len(cv_scores) == 4
        assert best_k in neighbors
        
        # Check values are within expected range
        assert all(0 <= score <= 1 for score in cv_scores)
    
    def test_invalid_k_range(self, test_data):
        """Test that evaluate_knn_cv raises appropriate errors for invalid input."""
        # Test with empty range
        with pytest.raises(ValueError, match="k_range cannot be empty"):
            evaluate_knn_cv(test_data['X_train'], test_data['y_train'], k_range=[])
        
        # Test with invalid type
        with pytest.raises(TypeError, match="k_range must be a range or list"):
            evaluate_knn_cv(test_data['X_train'], test_data['y_train'], k_range=42)


class TestPlotKnnCV:
    def test_basic_functionality(self):
        """Test that plot_knn_cv creates and returns a figure."""
        neighbors = [1, 2, 3, 4, 5]
        cv_scores = [0.8, 0.85, 0.9, 0.85, 0.8]
        
        fig = plot_knn_cv(neighbors, cv_scores)
        
        assert isinstance(fig, plt.Figure)
        plt.close(fig)  # Clean up
    
    def test_save_to_file(self):
        """Test that plot_knn_cv saves to file when output_path is provided."""
        neighbors = [1, 2, 3, 4, 5]
        cv_scores = [0.8, 0.85, 0.9, 0.85, 0.8]
        
        with tempfile.TemporaryDirectory() as tmp_dir:
            output_path = os.path.join(tmp_dir, 'test_plot.png')
            fig = plot_knn_cv(neighbors, cv_scores, output_path=output_path)
            
            # Check that file was created
            assert os.path.exists(output_path)
            plt.close(fig)  # Clean up
    
    def test_mismatched_lengths(self):
        """Test that plot_knn_cv raises error when inputs have different lengths."""
        neighbors = [1, 2, 3, 4, 5]
        cv_scores = [0.8, 0.85, 0.9]
        
        with pytest.raises(ValueError, match="neighbors and cv_scores must have the same length"):
            plot_knn_cv(neighbors, cv_scores)


class TestTrainKnnModel:
    def test_basic_functionality(self, test_data):
        """Test that train_knn_model returns a trained KNN model."""
        model = train_knn_model(test_data['X_train'], test_data['y_train'], k=3)
        
        assert isinstance(model, KNeighborsClassifier)
        assert model.n_neighbors == 3
        assert hasattr(model, 'predict')
        
        # Test that the model can make predictions
        predictions = model.predict(test_data['X_test'])
        assert len(predictions) == len(test_data['X_test'])
    
    def test_invalid_k(self, test_data):
        """Test that train_knn_model raises appropriate errors for invalid k."""
        # Test with non-integer k
        with pytest.raises(ValueError, match="k must be a positive integer"):
            train_knn_model(test_data['X_train'], test_data['y_train'], k=3.5)
        
        # Test with non-positive k
        with pytest.raises(ValueError, match="k must be a positive integer"):
            train_knn_model(test_data['X_train'], test_data['y_train'], k=0)


class TestEvaluateModel:
    def test_basic_functionality(self, test_data):
        """Test that evaluate_model returns the expected structure."""
        # Train a simple model
        model = KNeighborsClassifier(n_neighbors=3)
        model.fit(test_data['X_train'], test_data['y_train'])
        
        # Evaluate the model
        results = evaluate_model(model, test_data['X_test'], test_data['y_test'])
        
        # Check that the result has all expected keys
        expected_keys = ['accuracy', 'confusion_matrix', 'classification_report', 'y_pred', 'figure']
        for key in expected_keys:
            assert key in results
        
        # Check types
        assert isinstance(results['accuracy'], float)
        assert isinstance(results['confusion_matrix'], np.ndarray)
        assert isinstance(results['classification_report'], pd.DataFrame)
        assert isinstance(results['y_pred'], np.ndarray)
        assert isinstance(results['figure'], plt.Figure)
        
        # Check that the accuracy is within the expected range
        assert 0 <= results['accuracy'] <= 1
        
        # Check that the confusion matrix has the right shape
        assert results['confusion_matrix'].shape == (2, 2)  # Binary classification
        
        # Close the figure to clean up
        plt.close(results['figure'])
    
    def test_save_to_file(self, test_data):
        """Test that evaluate_model saves to file when output_path is provided."""
        # Train a simple model
        model = KNeighborsClassifier(n_neighbors=3)
        model.fit(test_data['X_train'], test_data['y_train'])
        
        with tempfile.TemporaryDirectory() as tmp_dir:
            output_path = os.path.join(tmp_dir, 'test_cm.png')
            results = evaluate_model(
                model, 
                test_data['X_test'], 
                test_data['y_test'],
                output_path=output_path
            )
            
            # Check that file was created
            assert os.path.exists(output_path)
            plt.close(results['figure'])  # Clean up


import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns
import os
from sklearn.neighbors import KNeighborsClassifier
from sklearn.model_selection import cross_val_score
from sklearn.metrics import confusion_matrix, classification_report
from banknote_utils import ensure_output_directory

def evaluate_knn_cv(X_train, y_train, k_range=range(1, 26)):
    """
    Evaluate different k values for KNN using cross-validation.
    
    Parameters
    ----------
    X_train : pandas.DataFrame or numpy.ndarray
        Training features.
    y_train : pandas.Series or numpy.ndarray
        Target variable for training.
    k_range : range or list, optional
        Range of k values to evaluate, by default range(1, 26).
    
    Returns
    -------
    tuple
        (neighbors, cv_scores, best_k) where:
        - neighbors is a list of k values evaluated
        - cv_scores is a list of corresponding mean CV scores
        - best_k is the k value with the highest CV score
    """
    if not isinstance(k_range, (range, list)):
        raise TypeError("k_range must be a range or list")
    if len(k_range) == 0:
        raise ValueError("k_range cannot be empty")
        
    neighbors = list(k_range)
    cv_scores = []
    
    for k in neighbors:
        knn = KNeighborsClassifier(n_neighbors=k)
        scores = cross_val_score(knn, X_train, y_train, cv=5)
        cv_scores.append(scores.mean())
    
    # Find the best k (maximum CV accuracy)
    best_k = neighbors[cv_scores.index(max(cv_scores))]
    
    return neighbors, cv_scores, best_k


def plot_knn_cv(neighbors, cv_scores, output_path=None, figsize=(10, 6)):
    """
    Plot the cross-validation accuracy vs. number of neighbors.
    
    Parameters
    ----------
    neighbors : list or array-like
        List of k values evaluated.
    cv_scores : list or array-like
        List of corresponding mean CV scores.
    output_path : str, optional
        Path to save the plot. If None, the plot is not saved.
    figsize : tuple, optional
        Figure size (width, height) in inches, by default (10, 6).
    
    Returns
    -------
    matplotlib.figure.Figure
        The created figure object.
    """
    if len(neighbors) != len(cv_scores):
        raise ValueError("neighbors and cv_scores must have the same length")
    
    plt.figure(figsize=figsize)
    fig = plt.gcf()
    plt.plot(neighbors, cv_scores, marker='o')
    plt.xlabel("Number of Neighbors")
    plt.ylabel("Mean CV Accuracy")
    plt.grid(True)
    
    if output_path:
        ensure_output_directory.ensure_output_directory(output_path)
            
        plt.savefig(output_path)
    
    return fig


def train_knn_model(X_train, y_train, k):
    """
    Train a KNN classifier with the specified k.
    
    Parameters
    ----------
    X_train : pandas.DataFrame or numpy.ndarray
        Training features.
    y_train : pandas.Series or numpy.ndarray
        Target variable for training.
    k : int
        Number of neighbors for KNN.
    
    Returns
    -------
    sklearn.neighbors.KNeighborsClassifier
        Trained KNN model.
    """
    if not isinstance(k, int) or k <= 0:
        raise ValueError("k must be a positive integer")
        
    knn = KNeighborsClassifier(n_neighbors=k)
    knn.fit(X_train, y_train)
    
    return knn


def evaluate_model(model, X_test, y_test, output_path=None, figsize=(6, 5)):
    """
    Evaluate a trained model and create a confusion matrix plot.
    
    Parameters
    ----------
    model : sklearn estimator
        Trained model with predict method.
    X_test : pandas.DataFrame or numpy.ndarray
        Test features.
    y_test : pandas.Series or numpy.ndarray
        True target values.
    output_path : str, optional
        Path to save the confusion matrix plot. If None, the plot is not saved.
    figsize : tuple, optional
        Figure size (width, height) in inches, by default (6, 5).
    
    Returns
    -------
    dict
        Dictionary containing:
        - 'accuracy': Test set accuracy
        - 'confusion_matrix': Confusion matrix
        - 'classification_report': DataFrame with the classification report
        - 'y_pred': Predicted values
    """
    # Generate predictions
    y_pred = model.predict(X_test)
    
    # Calculate metrics
    test_accuracy = model.score(X_test, y_test)
    cm = confusion_matrix(y_test, y_pred)
    report = pd.DataFrame(classification_report(y_test, y_pred, output_dict=True)).transpose()
    
    # Plot confusion matrix
    plt.figure(figsize=figsize)
    fig = plt.gcf()
    sns.heatmap(cm, annot=True, fmt="d", cmap="Blues")
    plt.xlabel("Predicted")
    plt.ylabel("Actual")
    plt.tight_layout()
    
    if output_path:
        ensure_output_directory.ensure_output_directory(output_path)
            
        plt.savefig(output_path)
    
    return {
        'accuracy': test_accuracy,
        'confusion_matrix': cm,
        'classification_report': report,
        'y_pred': y_pred,
        'figure': fig
    }


import os

def ensure_output_directory(output_prefix):
    """
    Ensure that the directory for the given output prefix exists.
    
    Parameters:
        output_prefix (str): The prefix for output files, including the directory path.
    
    Raises:
        FileExistsError: If a file exists at the expected directory location.
    """
    out_dir = os.path.dirname(output_prefix)

    if out_dir:
        # Check if something exists at this path
        if os.path.exists(out_dir) and not os.path.isdir(out_dir):  # If it's a file, raise an error
            raise FileExistsError(f"A file exists at the expected directory path: {out_dir}")

        os.makedirs(out_dir, exist_ok=True)  # Create directory if it doesn't exist



import click
import pandas as pd 

def check_missing_value(bill_data):
        
    """
    Checks if the data haas missing (NAN) values
    
    Parameters:
        “bill_data: a variable that stores a pandas DataFrame — which is a data table loaded from a CSV file.
    
    Output:
       If the dataset has NAN values, function prints a message in the terminal and filters the data to remove the NAN values

    """
        # Check if data has the right type
    if not isinstance(bill_data, pd.DataFrame):
        raise TypeError("Input must be a pandas DataFrame.")
    
    # Check for missing values
    missing_values = bill_data.isna().sum().sum()

    if missing_values:
        click.echo(f"Found {missing_values} missing values in the data.")
        # Drop missing values
        bill_data.dropna(inplace=True) #drops any rows with Nan
        click.echo("Missing values dropped!")

        return bill_data


    return bill_data


import pandas as pd
import matplotlib.pyplot as plt
from matplotlib.testing.compare import compare_images
from matplotlib import image
import os
import tempfile
import sys
import pytest
from sklearn.datasets import make_classification

# Add the src directory to the Python path
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

# Import the functions to test
from banknote_utils.visualization_utils import (plot_histogram, create_count_table)

# Create a fixture for test data
@pytest.fixture
def sample_data():
    X, y = make_classification(
        n_samples=100,
        n_features=4,
        n_informative=2,
        random_state=42
    )
    df = pd.DataFrame(X, columns=['feature1', 'feature2', 'feature3', 'feature4'])
    df['class'] = y
    return df

# get directory for the sample test images
@pytest.fixture
def get_test_images_dir():
    test_dir = os.path.join(os.path.dirname(__file__), 'test_images/')
    return test_dir

# tests for histogram plotting function
class TestPlotHistogram:
    # test if histogram compiles
    def test_plot_histogram_basic(self, sample_data):
        feature = 'feature1'
        plot_histogram(sample_data, feature, "class")
        
        # use unknown_variable as target variable instead of 'class'
        sample_data.rename(columns={'class': 'unknown_variable'}, inplace=True)
        plot_histogram(sample_data, feature, "unknown_variable")

    # test if histogram returns an error if feature or classes are missing from dataset
    def test_plot_histogram_error(self, sample_data):

        # missing feature
        missing_feature = 'non_existent_feature'
        with pytest.raises(ValueError, match="Could not interpret value `non_existent_feature` for `x`. An entry with this name does not appear in `data`"):
            plot_histogram(sample_data, missing_feature, "class")

        # missing target_variable
        feature = 'feature1'
        with pytest.raises(ValueError, match="Could not interpret value `missing_class` for `hue`. An entry with this name does not appear in `data`"):
            plot_histogram(sample_data, feature, "missing_class")

        # empty dataframe case
        empty_df = pd.DataFrame(columns=["feature1", 'lacking_class'])
        with pytest.raises(ValueError, match="`dataset` input should have multiple elements."):
            plot_histogram(empty_df, feature, "lacking_class")

        # dataframe with only one item
        single_item_df = pd.DataFrame({"feature1": [1.0], "lacking_class": ["A"]})
        with pytest.raises(ValueError, match="`dataset` input should have multiple elements."):
            plot_histogram(single_item_df, feature, "lacking_class")

    # test if histogram saves to directory and matches test image
    def test_plot_histogram_with_saving(self, sample_data, get_test_images_dir):
        feature = 'feature1'
        resulting_image = f"{get_test_images_dir}with_saving_figure_{feature}.png"
        
        with tempfile.TemporaryDirectory() as temp_dir:
            output_prefix = os.path.join(temp_dir, 'with_saving_figure')
            plot_histogram(sample_data, feature, "class", output_prefix=output_prefix)
            generated_image = f"{output_prefix}_{feature}.png"
            assert os.path.exists(generated_image), f"Output file {generated_image} was not created"
            diff = compare_images(resulting_image, generated_image, tol=1e-2)
            assert diff is None, f"Test failed: Images are different when should have the same default parameters: {diff}"

    # test if histogram with custom labels matches test image
    def test_plot_histogram_with_labels(self, sample_data, get_test_images_dir):
        feature = 'feature1'
        labels = ['A (Tan Bars)', 'B (Blue Bars)']
        labels2 = ["These labels shouldn't match at all", 'As they are completely different']
        resulting_image = f"{get_test_images_dir}with_labels_{feature}.png"
        
        with tempfile.TemporaryDirectory() as temp_dir:
            output_prefix = os.path.join(temp_dir, 'with_labels')

            # test matching labels
            plot_histogram(sample_data, feature, "class", labels=labels,  output_prefix=output_prefix)
            generated_image = f"{output_prefix}_{feature}.png"
            assert os.path.exists(generated_image), f"Output file {generated_image} was not created"
            diff = compare_images(resulting_image, generated_image, tol=0)
            assert diff is None, f"Test failed: Images are different when they should have the same label: {diff}"

            # test differing labels
            plot_histogram(sample_data, feature, "class", labels=labels2,  output_prefix=output_prefix)
            diff = compare_images(resulting_image, generated_image, tol=0)
            assert diff is not None, f"Test failed: Images are the same when they should differ in labels"

    # test if histogram with custom figsizes matches test image
    def test_plot_histogram_with_figsize(self, sample_data, get_test_images_dir):
        feature = 'feature1'
        labels = ['A (Tan Bars)', 'B (Blue Bars)']
        figsize = (10, 10)
        resulting_image = f"{get_test_images_dir}with_figsize_{feature}.png"
        
        with tempfile.TemporaryDirectory() as temp_dir:
            output_prefix = os.path.join(temp_dir, 'with_labels')

            # test matching figsizes
            plot_histogram(sample_data, feature, "class", labels=labels,  output_prefix=output_prefix, figsize=figsize)
            generated_image = f"{output_prefix}_{feature}.png"
            assert os.path.exists(generated_image), f"Output file {generated_image} was not created"
            diff = compare_images(resulting_image, generated_image, tol=0)
            assert diff is None, f"Test failed: Images are different when they should have the same figure size: {diff}"
            
            # test differing figsizes
            plot_histogram(sample_data, feature, "class", labels=labels,  output_prefix=output_prefix)
            img1 = image.imread(resulting_image)
            img2 = image.imread(generated_image)
            assert img1.shape != img2.shape, "Test failed: Images have the same sizes when they should have different figure sizes"

# tests for create_count_table function
class TestCreateCountTable:
    # test if count table is created & compiles
    def test_basic_count_table(self, sample_data):
        create_count_table(sample_data, "class")

    # test if target class exists and returns error if not
    def test_count_table_error(self, sample_data):
        wrong_class_name = "wrong_target"
        
        with pytest.raises(KeyError, match=wrong_class_name):
            create_count_table(sample_data, wrong_class_name)

    # test whether count table saves correctly
    def test_count_table_with_saving(self, sample_data):
        with tempfile.TemporaryDirectory() as temp_dir:
            # check if table has 2 classes & 3 coulmns denoting count table information
            create_count_table(sample_data, "class", temp_dir)
            generated_table = f"{temp_dir}_count_table.csv"
            assert os.path.exists(generated_table), f"Output file {generated_table} was not created"
            table = pd.read_csv(generated_table)
            assert table.shape == (2, 3), f"Shape mismatch: Expected (2, 3), but got {table.shape}"

            # create a new row & check if the newly created table has 3 classes
            new_row = {'feature1': 1, 'feature2': 2, 'feature3': 3, 'feature4': 4, 'class': 30}
            new_row_df = pd.DataFrame([new_row])
            additional_class_sample = pd.concat([sample_data, new_row_df], ignore_index=True)
            create_count_table(additional_class_sample, "class", temp_dir)
            additional_class_table =  pd.read_csv(generated_table)
            assert additional_class_table.shape == (3, 3), f"Shape mismatch: Expected (3, 3), but got {additional_class_table.shape}"

            # check case containing no rows
            empty_df = pd.DataFrame(columns=['lacking_class'])
            create_count_table(empty_df, "lacking_class", temp_dir)
            empty_class_table = pd.read_csv(generated_table)
            assert empty_class_table.shape == (0, 3), f"Shape mismatch: Expected (0, 3), but got {empty_class_table.shape}"




import os
import pytest
import sys

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
from banknote_utils.ensure_output_directory import ensure_output_directory

@pytest.fixture
def temp_output_prefix(tmp_path):
    """Provides a temporary directory for testing."""
    return os.path.join(tmp_path, "test_output")

def test_creates_new_directory(temp_output_prefix):
    """Test that the function creates a directory when it doesn't exist."""
    ensure_output_directory(temp_output_prefix)
    assert os.path.exists(os.path.dirname(temp_output_prefix))

def test_existing_directory(temp_output_prefix):
    """Test that the function does nothing if the directory already exists."""
    dir_path = os.path.dirname(temp_output_prefix)

    # Ensure directory exists before calling function
    os.makedirs(dir_path, exist_ok=True)  
    assert os.path.isdir(dir_path)  # Ensure it's actually a directory

    ensure_output_directory(temp_output_prefix)
    assert os.path.exists(dir_path)  # Directory should still exist

def test_empty_prefix():
    """Test that the function does nothing when given an empty string."""
    ensure_output_directory("")  # Should not raise an error

def test_conflicting_file(tmp_path):
    """Test that the function raises an error if a file exists at the expected directory path."""
    conflicting_dir_path = tmp_path / "conflicting_dir"

    # Create a file where the directory should be
    conflicting_dir_path.write_text("This is a file, not a directory.")

    # Use a fake output prefix that expects conflicting_dir_path to be a directory
    output_prefix = str(conflicting_dir_path / "output_file.csv")

    with pytest.raises(FileExistsError, match="A file exists at the expected directory path"):
        ensure_output_directory(output_prefix)  # Should raise an error

def test_root_directory(tmp_path):
    """Test that the function does nothing if the output is in a root directory."""
    output_prefix = str(tmp_path / "output_file.csv")  # Use tmp_path as root dir

    ensure_output_directory(output_prefix)  # Should NOT raise an error

    assert tmp_path.exists() and tmp_path.is_dir()  # Root dir should still exist


import os
import pytest
import sys
import pandas as pd
import numpy as np

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
from banknote_utils.check_missing_value import check_missing_value

@pytest.fixture
# create test clean dataset 
def sample_df_no_missing():
    return pd.DataFrame({
        "name": ["Alice", "Bob"],
        "amount": [25.5, 30.0]
    })

@pytest.fixture
# create test dataset with one missing value 
def sample_df_with_missing():
    return pd.DataFrame({
        "name": ["Alice", "Bob", "Charlie"],
        "amount": [25.5, None, 30.0]
    })

@pytest.fixture
#create test dataset with all missing values
def sample_df_with_all_rows_missing():
    return pd.DataFrame({
        "name": [None, None, None],
        "amount": [None, None, None]
    })

@pytest.fixture
#create test dataset with all missing values
def sample_not_df():
    return np.array([1,2,3])

#test if the dataframe is clean
def test_no_missing_value(sample_df_no_missing,capfd):
    result = check_missing_value(sample_df_no_missing.copy())
    assert result.equals(sample_df_no_missing)

    out, _ = capfd.readouterr()
    assert "missing values" not in out.lower()


#test if the dataframe has one row with missign value
def test_one_missing_value(sample_df_with_missing,capfd):
    result = check_missing_value(sample_df_with_missing.copy())
    out, _ = capfd.readouterr()

    assert result.shape[0] == 2 # one row should be dropped
    assert result["amount"].isna().sum() == 0  # No NaNs left
    assert "missing values" in out.lower()
    assert "Missing values dropped!" in out #Make sure it prints out the right message in the terminal

#test if the dataframe has all rows missing
def test_all_missing_value(sample_df_with_all_rows_missing,capfd):

    result = check_missing_value(sample_df_with_all_rows_missing.copy())
    out, _ = capfd.readouterr()

    assert result.empty
    assert "missing values" in out.lower()
    assert "dropped!" in out.lower()

#test if it has the right data type 
def test_wrong_type(sample_not_df):
       
       with pytest.raises(TypeError):
           check_missing_value(sample_not_df)





import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns
from banknote_utils import ensure_output_directory

def plot_histogram(dataset, feature, target_variable, labels=None, output_prefix=None, figsize=(8, 6)):
    """
    Creates a basic histogram for the given dataset using the given feature
    
    Parameters
    ----------
    dataset : pandas.DataFrame or numpy.ndarray
        DataFrame containing the given feature to plot a histogram for
    feature : pandas.Series or numpy.ndarray
        Target variable to plot a histogram for
    labels : list of strings, optional
        Add custom labels for the histogram's legend. If None, use default labels.
    output_path : str, optional
        Path to save the histogram. If None, the plot is not saved.
    figsize : tuple, optional
        Figure size (width, height) in inches, by default (8, 6).
    
    Returns
    -------
    None

    If a output_prefix is given, the function saves a plot to the given file location
    """
    plt.figure(figsize=figsize)
    sns.histplot(data=dataset, x=feature, hue=target_variable, element='bars', bins=30, kde=True)
    plt.xlabel(feature.capitalize())
    plt.ylabel("Count")
    if labels:
        plt.legend(title=target_variable.capitalize(), labels=labels)
    plt.tight_layout()
    if output_prefix:
        ensure_output_directory.ensure_output_directory(output_prefix)
        plt.savefig(f"{output_prefix}_{feature}.png")
    plt.close()


def create_count_table(dataset, target_variable, output_prefix=None):
    """
    Creates a count table showing the total amount and proportion of each class
    for the given dataset using the given target variable
    
    Parameters
    ----------
    dataset : pandas.DataFrame or numpy.ndarray
        DataFrame containing the given feature to plot a histogram for
    target_variable : pandas.Series or numpy.ndarray
        Target variable to plot a histogram for
    output_path : str, optional
        Path to save the count table. If None, the plot is not saved.
    
    Returns
    -------
    None

    If an output_prefix is given, the function saves the count table to the given file location
    """
    count_table = dataset.groupby(target_variable).size().reset_index(name='Count')
    count_table['Percentage'] = 100 * count_table['Count'] / len(dataset)
    if output_prefix:
        ensure_output_directory.ensure_output_directory(output_prefix)
        count_table.to_csv(f"{output_prefix}_count_table.csv", index=False)



"""
Banknote Utils
==============

A Python package for banknote classification and analysis utilities, providing tools for data processing,
visualization, and modeling focused on banknote authentication.

This package includes functions for:
- Checking and handling missing values in datasets
- Ensuring output directories exist
- K-Nearest Neighbors (KNN) modeling utilities
- Visualization utilities for exploratory data analysis
"""

__version__ = "0.1.6"

# Import functions from modules
from .check_missing_value import check_missing_value
from .ensure_output_directory import ensure_output_directory
from .modeling_utils import (
    evaluate_knn_cv,
    plot_knn_cv,
    train_knn_model,
    evaluate_model
)
from .visualization_utils import (
    create_count_table,
    create_histogram
)

__all__ = [
    "check_missing_value",
    "ensure_output_directory",
    "evaluate_knn_cv",
    "plot_knn_cv",
    "train_knn_model",
    "evaluate_model",
    "create_count_table",
    "create_histogram"
]